import * as path from 'path';
import path__default from 'path';
import { cosmiconfigSync } from 'cosmiconfig';
import jsStringEscape from 'js-string-escape';
import deepmerge from 'deepmerge';
import { hasProperties, PropKind, isUnionProp, isStringProp, isClassLikeProp, isEnumProp, isArrayProp, isTupleProp, isIndexProp, isFunctionProp } from '@structured-types/api';
import * as fs from 'fs';
import hostedGitInfo from 'hosted-git-info';
import parseRepositoryURL from '@hutson/parse-repository-url';

/**
 * Documentation node kinds
 */
var NodeKind;
(function (NodeKind) {
    NodeKind[NodeKind["Table"] = 1] = "Table";
    NodeKind[NodeKind["TableRow"] = 2] = "TableRow";
    NodeKind[NodeKind["TableCell"] = 3] = "TableCell";
    NodeKind[NodeKind["Heading"] = 4] = "Heading";
    NodeKind[NodeKind["Paragraph"] = 5] = "Paragraph";
    NodeKind[NodeKind["Text"] = 6] = "Text";
    NodeKind[NodeKind["Bold"] = 7] = "Bold";
    NodeKind[NodeKind["Emphasis"] = 8] = "Emphasis";
    NodeKind[NodeKind["Link"] = 9] = "Link";
    NodeKind[NodeKind["Code"] = 10] = "Code";
    NodeKind[NodeKind["InlineCode"] = 11] = "InlineCode";
})(NodeKind || (NodeKind = {}));
/**
 * Node with children type guard predicate
 */
const isNodeWithChildren = (node) => {
    return (node.kind === NodeKind.Table ||
        node.kind === NodeKind.TableRow ||
        node.kind === NodeKind.TableCell ||
        node.kind === NodeKind.Heading ||
        node.kind === NodeKind.Paragraph ||
        node.kind === NodeKind.Bold ||
        node.kind === NodeKind.Emphasis ||
        node.kind === NodeKind.Link);
};
/**
 * Node with value type guard predicate
 */
const isNodeWithValue = (node) => {
    return (node.kind === NodeKind.Text ||
        node.kind === NodeKind.InlineCode ||
        node.kind === NodeKind.Code);
};
/**
 * TableNode type guard predicate
 */
const isTableNode = (node) => {
    return node.kind === NodeKind.Table;
};
/**
 * TableRowNode type guard predicate
 */
const isTableRowNode = (node) => {
    return node.kind === NodeKind.TableRow;
};
/**
 * TableCellNode type guard predicate
 */
const isTableCellNode = (node) => {
    return node.kind === NodeKind.TableCell;
};
/**
 * HeadingNode type guard predicate
 */
const isHeadingNode = (node) => {
    return node.kind === NodeKind.Heading;
};
/**
 * ParagraphNode type guard predicate
 */
const isParagraphNode = (node) => {
    return node.kind === NodeKind.Paragraph;
};
/**
 * TextNode type guard predicate
 */
const isTextNode = (node) => {
    return node.kind === NodeKind.Text;
};
/**
 * BoldNode type guard predicate
 */
const isBoldNode = (node) => {
    return node.kind === NodeKind.Bold;
};
/**
 * EmphasisNode type guard predicate
 */
const isEmphasisNode = (node) => {
    return node.kind === NodeKind.Emphasis;
};
/**
 * LinkNode type guard predicate
 */
const isLinkNode = (node) => {
    return node.kind === NodeKind.Link;
};
/**
 * CodeNode type guard predicate
 */
const isCodeNode = (node) => {
    return node.kind === NodeKind.Code;
};
/**
 * InlineCodeNode type guard predicate
 */
const isInlineCodeNode = (node) => {
    return node.kind === NodeKind.InlineCode;
};

const traverseFolder = (filePath, levels = 10, fileName = 'package.json') => {
    if (!fs.existsSync(filePath)) {
        return null;
    }
    const files = fs.readdirSync(filePath);
    if (levels === 0) {
        return null;
    }
    const pckg = files.find((file) => file === fileName);
    if (pckg) {
        return path__default.resolve(filePath, pckg);
    }
    return traverseFolder(path__default.resolve(filePath, '..'), levels - 1, fileName);
};
/**
 * Retrieves the repo path from the project's package.json file.
 * @param filePath file path to start the search for a package.json
 */
const getRepoPath = (filePath) => {
    const packageFileName = traverseFolder(path__default.dirname(filePath));
    const result = {};
    if (packageFileName) {
        const packageJSON = JSON.parse(fs.readFileSync(packageFileName, 'utf8'));
        result.packageName = packageJSON.name;
        result.relativePath = path__default.relative(path__default.dirname(packageFileName), filePath);
        const repositoryURL = typeof packageJSON.repository === 'string'
            ? packageJSON.repository
            : packageJSON.repository && packageJSON.repository.url;
        if (repositoryURL) {
            const templates = hostedGitInfo.fromUrl(repositoryURL) ||
                parseRepositoryURL(repositoryURL);
            result.relativePath = `${packageJSON.repository && packageJSON.repository.directory
                ? `${packageJSON.repository.directory}/`
                : ''}${path__default.relative(path__default.dirname(packageFileName), filePath)}`;
            const fillTemplate = (template) => {
                return template
                    .replace('auth@', templates.auth || '')
                    .replace('{domain}', templates.domain || '')
                    .replace('{user}', templates.user || '')
                    .replace('{project}', templates.project || '')
                    .replace('{#fragment}', '')
                    .replace('{path}', result.relativePath)
                    .replace('{treepath}', templates.treepath || '')
                    .replace('{/tree/committish}', `/${templates.treepath}/${templates.committish || 'master'}`)
                    .replace('{committish}', templates.committish || 'master');
            };
            result.repo = fillTemplate(templates.browsefiletemplate);
        }
    }
    return result;
};

const textNode = (value) => ({ kind: NodeKind.Text, value });

const tableCellNode = (value) => ({
    kind: NodeKind.TableCell,
    children: typeof value === 'string' ? [textNode(value)] : value,
});

const inlineCodeNode = (value) => ({
    kind: NodeKind.InlineCode,
    value,
});

const createPropsRow = ({ name, type, value, parent, description }, visColumns) => {
    const children = [
        tableCellNode(name || []),
        tableCellNode(type || []),
    ];
    if (visColumns.parents) {
        children.push(tableCellNode(parent || []));
    }
    if (visColumns.values) {
        children.push(tableCellNode([
            inlineCodeNode(typeof value !== 'undefined' ? value.toString() : ''),
        ]));
    }
    if (visColumns.descriptions) {
        if (description) {
            const parts = description.split('`');
            if (parts.length > 1) {
                children.push(tableCellNode(parts.reduce((acc, text, idx) => {
                    if (idx % 2 === 0) {
                        return [...acc, textNode(text)];
                    }
                    else {
                        return [
                            ...acc,
                            textNode(' '),
                            inlineCodeNode(text),
                            textNode(' '),
                        ];
                    }
                }, [])));
            }
            else {
                children.push(tableCellNode([textNode(description)]));
            }
        }
        else {
            children.push(tableCellNode([textNode('')]));
        }
    }
    return {
        kind: NodeKind.TableRow,
        children,
    };
};
const createPropsTable = (children) => {
    var _a;
    const propsTable = [];
    let table = undefined;
    const visibleColumns = {};
    if (children) {
        const columns = [];
        visibleColumns.names = children.some((item) => item.name !== undefined);
        if (visibleColumns.names) {
            columns.push(tableCellNode([textNode('Name')]));
        }
        visibleColumns.types = children.some((item) => item.type !== undefined);
        if (visibleColumns.types) {
            columns.push(tableCellNode([textNode('Type')]));
        }
        visibleColumns.parents = children.some((item) => item.parent !== undefined);
        if (visibleColumns.parents) {
            columns.push(tableCellNode([textNode('Parent')]));
        }
        visibleColumns.values = children.some((item) => item.value !== undefined);
        if (visibleColumns.values) {
            columns.push(tableCellNode([textNode('Default')]));
        }
        visibleColumns.descriptions = children.some((item) => item.description);
        if (visibleColumns.descriptions) {
            columns.push(tableCellNode([textNode('Description')]));
        }
        table = {
            kind: NodeKind.Table,
            children: [
                {
                    kind: NodeKind.TableRow,
                    children: columns,
                },
            ],
        };
        propsTable.push(table);
        // eslint-disable-next-line prefer-spread
        (_a = table.children) === null || _a === void 0 ? void 0 : _a.push(...children.map((child) => createPropsRow(child, visibleColumns)));
    }
    return { propsTable: propsTable, table, visibleColumns };
};

const paragraphNode = (value) => ({
    kind: NodeKind.Paragraph,
    children: typeof value === 'string' ? [textNode(value)] : value,
});

const emphasisNode = (value) => ({
    kind: NodeKind.Emphasis,
    children: typeof value === 'string' ? [textNode(value)] : value,
});

const codeNode = (value) => ({
    kind: NodeKind.Code,
    value,
});

const linkNode = (children, url, loc) => {
    const prop = {
        kind: NodeKind.Link,
        children,
    };
    if (url) {
        prop.url = url;
    }
    if (loc) {
        prop.loc = loc;
    }
    return prop;
};

const headingNode = (value, depth) => ({
    kind: NodeKind.Heading,
    depth,
    children: typeof value === 'string' ? [textNode(value)] : value,
});

const boldNode = (value) => ({
    kind: NodeKind.Bold,
    children: typeof value === 'string' ? [textNode(value)] : value,
});

const defaultSections = {
    title: {},
    type: {},
    description: {},
    location: {},
    props: { title: 'properties' },
    examples: {
        title: (prop) => prop.examples && prop.examples.length > 1 ? 'examples' : 'example',
    },
};

/* eslint-disable prefer-spread */
class PropsToDocumentation {
    constructor() {
        this.collapsed = [];
        this.sections = defaultSections;
        this.columns = ['all'];
        this.skipInherited = false;
        this.topLevelProps = {};
        this.helpers = {};
        this.repoNames = {};
        this.getPropLink = (key) => {
            const nameParts = key.split('.');
            return this.topLevelProps[nameParts[nameParts.length - 1]];
        };
    }
    extractPropTable(props) {
        let parentProp = undefined;
        const addParentProp = (prop) => {
            var _a, _b;
            if (prop.parent) {
                if (!parentProp) {
                    parentProp = {
                        name: '...props',
                        kind: PropKind.Enum,
                        properties: [
                            {
                                kind: PropKind.Type,
                                type: prop.parent.name,
                                loc: prop.parent.loc,
                            },
                        ],
                        optional: true,
                    };
                }
                else {
                    if (!((_a = parentProp.properties) === null || _a === void 0 ? void 0 : _a.find((p) => { var _a; return p.type === ((_a = prop.parent) === null || _a === void 0 ? void 0 : _a.name); }))) {
                        (_b = parentProp.properties) === null || _b === void 0 ? void 0 : _b.push({
                            kind: PropKind.Type,
                            type: prop.parent.name,
                            loc: prop.parent.loc,
                        });
                    }
                }
            }
        };
        const consolidatedProps = props.filter((prop) => {
            var _a;
            if (prop.parent) {
                if (this.skipInherited || this.collapsed.includes(prop.parent.name)) {
                    addParentProp(prop);
                    return false;
                }
                for (const collapsedProp of this.collapsed) {
                    const helperParent = this.helpers[collapsedProp];
                    if (helperParent && hasProperties(helperParent)) {
                        const helpProp = (_a = helperParent.properties) === null || _a === void 0 ? void 0 : _a.find((p) => { var _a, _b; return p.name === prop.name && ((_a = p.parent) === null || _a === void 0 ? void 0 : _a.name) === ((_b = prop.parent) === null || _b === void 0 ? void 0 : _b.name); });
                        if (helpProp) {
                            addParentProp({
                                ...prop,
                                parent: { name: collapsedProp, loc: helperParent.loc },
                            });
                            return false;
                        }
                    }
                }
            }
            return true;
        });
        const allProps = parentProp
            ? [...consolidatedProps, parentProp]
            : consolidatedProps;
        const items = allProps.map((prop) => {
            let name = prop.name
                ? prop.kind === PropKind.Rest
                    ? `...${prop.name}`
                    : prop.name
                : '';
            name = `${name}${prop.optional ? '' : '*'}`;
            return this.configurePropItem({
                name: prop.loc
                    ? this.propLink({ name, loc: prop.loc })
                    : [inlineCodeNode(name)],
                parent: prop.parent ? this.propLink(prop.parent) : undefined,
                type: this.extractPropType(prop, { extractProperties: true }),
                description: prop.description,
                value: prop.value,
            });
        });
        return createPropsTable(items);
    }
    extractFunctionDeclaration(prop) {
        const result = [textNode('(')];
        if (prop.parameters) {
            for (let i = 0; i < prop.parameters.length; i += 1) {
                const p = prop.parameters[i];
                if (i > 0) {
                    result.push(textNode(', '));
                }
                if (p.name) {
                    result.push(inlineCodeNode(p.name));
                    if (!p.optional) {
                        result.push(textNode('*'));
                    }
                    result.push(textNode(': '));
                }
                result.push(...this.extractPropType(p));
                if (!p.name && !p.optional) {
                    result.push(textNode('*'));
                }
            }
        }
        result.push(textNode(')'));
        result.push(textNode(' => '));
        if (prop.returns) {
            result.push(...this.extractPropType(prop.returns));
        }
        else {
            result.push(textNode('void'));
        }
        return result;
    }
    configurePropItem(item) {
        const enabledColumn = (name) => {
            return this.columns.includes(name) || this.columns.includes('all');
        };
        return {
            name: enabledColumn('name') ? item.name : undefined,
            parent: enabledColumn('parents') ? item.parent : undefined,
            type: enabledColumn('type') ? item.type : undefined,
            value: enabledColumn('default') ? item.value : undefined,
            description: enabledColumn('description') ? item.description : undefined,
        };
    }
    extractFunction(prop, _extractTable = true) {
        if (prop.parameters) {
            const { propsTable, table, visibleColumns } = this.extractPropTable(prop.parameters);
            if (table &&
                table.children &&
                prop.returns &&
                prop.returns.kind !== PropKind.Void) {
                table.children.push(createPropsRow(this.configurePropItem({
                    name: [inlineCodeNode('returns')],
                    parent: prop.returns.parent
                        ? this.propLink(prop.returns.parent)
                        : undefined,
                    type: this.extractPropType(prop.returns),
                    description: prop.returns.description,
                }), visibleColumns));
            }
            return propsTable;
        }
        return undefined;
    }
    extractInterface(prop) {
        const result = [];
        if (prop.name) {
            if (isUnionProp(prop)) {
                result.push(...this.extractPropType(prop));
            }
            else if (hasProperties(prop) && prop.properties) {
                const { propsTable } = this.extractPropTable(prop.properties);
                result.push(...propsTable);
            }
        }
        return result;
    }
    propLink(prop) {
        var _a;
        const typeText = [inlineCodeNode(prop.name)];
        if (typeof prop.name === 'string') {
            const link = this.getPropLink(prop.name);
            return [
                linkNode(typeText, link ? `#${(_a = link.name) === null || _a === void 0 ? void 0 : _a.toLowerCase()}` : undefined, prop.loc),
            ];
        }
        return typeText;
    }
    inlineType(prop) {
        let typeNode = undefined;
        if (typeof prop.type === 'string') {
            typeNode = this.propLink({ name: prop.type, loc: prop.loc });
        }
        else if (prop.kind) {
            typeNode = [inlineCodeNode(`${PropKind[prop.kind].toLowerCase()}`)];
        }
        if (prop.name && prop.name !== prop.type) {
            return [
                inlineCodeNode(`${prop.name}`),
                textNode(`${typeNode ? ': ' : ''}`),
                ...(typeNode || []),
            ];
        }
        return typeNode || [];
    }
    typeNode(prop, showValue = true) {
        if (typeof prop.type === 'string') {
            if (typeof prop.parent === 'string') {
                return [...this.propLink(prop.parent), textNode(`.${prop.type}`)];
            }
            return this.propLink({ name: prop.type, loc: prop.loc });
        }
        if (showValue && prop.value !== undefined) {
            const value = isStringProp(prop)
                ? `"${jsStringEscape(prop.value)}"`
                : prop.value.toString();
            return [textNode(value)];
        }
        if (prop.kind) {
            const typeNode = [
                inlineCodeNode(`${PropKind[prop.kind].toLowerCase()}`),
            ];
            if (typeof prop.parent === 'string' && this.getPropLink(prop.parent)) {
                const link = this.propLink(prop.parent);
                if (link.length) {
                    typeNode.push(textNode(` (`));
                    typeNode.push(...link);
                    typeNode.push(textNode(`)`));
                }
            }
            return typeNode;
        }
        if (prop.name) {
            return [textNode(prop.name)];
        }
        return [];
    }
    extractPropType(prop, options) {
        var _a;
        if (prop.parent) {
            const parent = this.getPropLink(prop.parent.name);
            if (parent && isClassLikeProp(parent)) {
                const p = (_a = parent.properties) === null || _a === void 0 ? void 0 : _a.find((p) => p.name === prop.name);
                if (p) {
                    return this.extractType(p, options);
                }
            }
        }
        return this.extractType(prop, options);
    }
    extractType(prop, options) {
        var _a, _b, _c, _d, _e;
        const { showValue = false } = options || {};
        if (typeof prop.type === 'string' && ((_a = this.collapsed) === null || _a === void 0 ? void 0 : _a.includes(prop.type))) {
            return this.typeNode(prop, showValue);
        }
        else if ((isUnionProp(prop) || isEnumProp(prop)) && prop.properties) {
            const separator = isUnionProp(prop) ? ' | ' : ' & ';
            return (_b = prop.properties) === null || _b === void 0 ? void 0 : _b.reduce((acc, t, idx) => {
                const r = [...acc, ...this.extractPropType(t, { showValue: true })];
                if (prop.properties && idx < prop.properties.length - 1) {
                    r.push(textNode(separator));
                }
                return r;
            }, []);
        }
        else if (isClassLikeProp(prop)) {
            const propName = typeof prop.type === 'string' ? prop.type : prop.name;
            const result = [];
            if (typeof propName === 'string' && this.getPropLink(propName)) {
                result.push(...this.propLink({ name: propName, loc: prop.loc }));
            }
            else if ((_c = prop.properties) === null || _c === void 0 ? void 0 : _c.length) {
                const typeArguments = prop.properties.reduce((acc, p, idx) => {
                    const result = [...acc, ...this.inlineType(p)];
                    if (prop.properties && idx < prop.properties.length - 1) {
                        result.push(textNode(', '));
                    }
                    return result;
                }, []);
                result.push(textNode('{ '));
                result.push(...typeArguments);
                result.push(textNode(' }'));
            }
            else if ((_d = prop.generics) === null || _d === void 0 ? void 0 : _d.length) {
                const genericArguments = (_e = prop.generics) === null || _e === void 0 ? void 0 : _e.reduce((acc, p, idx) => {
                    const result = [...acc, ...this.inlineType(p)];
                    if (prop.generics && idx < prop.generics.length - 1) {
                        result.push(textNode(', '));
                    }
                    return result;
                }, []);
                result.push(...this.typeNode(prop));
                if (genericArguments === null || genericArguments === void 0 ? void 0 : genericArguments.length) {
                    result.push(...[textNode('<'), ...genericArguments, textNode('>')]);
                }
            }
            else if (propName) {
                result.push(textNode(propName));
            }
            return result;
        }
        else if (isArrayProp(prop) && prop.properties) {
            const elements = prop.properties.reduce((acc, p, idx) => {
                const result = this.extractPropType(p);
                if (prop.properties && idx < prop.properties.length - 1) {
                    result.push(textNode(', '));
                }
                return [...acc, ...result];
            }, []);
            const multiProps = elements.length &&
                elements[0].children &&
                elements[0].children.length > 1;
            if (multiProps) {
                elements.splice(0, 0, textNode('('));
                elements.push(textNode(')'));
            }
            elements.push(textNode('[]'));
            return elements;
        }
        else if (isTupleProp(prop) && prop.properties) {
            return [
                textNode('['),
                ...prop.properties.reduce((acc, p, idx) => {
                    const result = this.extractPropType(p);
                    if (prop.properties && idx < prop.properties.length - 1) {
                        result.push(textNode(', '));
                    }
                    return [...acc, ...result];
                }, []),
                { kind: NodeKind.Text, value: ']' },
            ];
        }
        else if (isIndexProp(prop)) {
            const results = [
                textNode('['),
                ...this.extractPropType(prop.index),
                textNode(']'),
            ];
            if (prop.prop) {
                results.push(textNode(': '));
                results.push(...this.extractPropType(prop.prop));
            }
            return results;
            //return this.extractFunctionDeclaration(prop);
        }
        else if (isFunctionProp(prop)) {
            return this.extractFunctionDeclaration(prop);
        }
        return this.typeNode(prop, showValue);
    }
    getSourceLocation(prop) {
        const { filePath, loc } = prop.loc || {};
        if (filePath) {
            if (!this.repoNames[filePath]) {
                this.repoNames[filePath] = getRepoPath(path__default.resolve(filePath));
            }
            if (this.repoNames[filePath]) {
                const { repo, relativePath, packageName } = this.repoNames[filePath];
                const fileLocation = repo || filePath;
                if (fileLocation) {
                    const { line } = (loc === null || loc === void 0 ? void 0 : loc.start) || {};
                    const sourceLocation = filePath.includes('node_modules')
                        ? fileLocation
                        : `${fileLocation}${line ? `#L${line}` : ''}`;
                    return [
                        paragraphNode([
                            emphasisNode([
                                textNode('defined in '),
                                linkNode([
                                    textNode(packageName
                                        ? `${packageName}/${relativePath}`
                                        : sourceLocation),
                                ], sourceLocation, prop.loc),
                            ]),
                        ]),
                    ];
                }
            }
        }
        return [];
    }
    getType(prop) {
        if (prop.kind) {
            return [
                boldNode([
                    inlineCodeNode(`${prop.extension ? `${prop.extension} ` : ''}${PropKind[prop.kind].toLowerCase()}`),
                ]),
            ];
        }
        else if (typeof prop.type === 'string') {
            return [
                headingNode(this.propLink({ name: prop.type, loc: prop.loc }), 3),
            ];
        }
        return undefined;
    }
    getLocation(prop) {
        const loc = this.getSourceLocation(prop);
        if (loc.length) {
            return loc;
        }
        return undefined;
    }
    getPropsTable(prop) {
        if (isFunctionProp(prop)) {
            return this.extractFunction(prop);
        }
        else if (hasProperties(prop)) {
            return this.extractInterface(prop);
        }
        return undefined;
    }
    getExamples(prop) {
        if (prop.examples) {
            const result = [];
            const codeExamples = prop.examples.filter((e) => e.content);
            codeExamples.forEach((example) => {
                if (example.content) {
                    result.push(paragraphNode([codeNode(example.content)]));
                }
            });
            return result;
        }
        return undefined;
    }
    getExtends(prop) {
        var _a;
        if (isClassLikeProp(prop) && ((_a = prop.extends) === null || _a === void 0 ? void 0 : _a.length)) {
            const extendsList = prop.extends.reduce((acc, key, idx) => {
                const p = this.getPropLink(key);
                let result;
                if (p) {
                    result = this.extractPropType(p);
                }
                else {
                    result = [textNode(key)];
                }
                if (prop.extends && idx < prop.extends.length - 1) {
                    result.push(textNode(', '));
                }
                return [...acc, ...result];
            }, []);
            return [textNode('extends '), ...extendsList];
        }
        return undefined;
    }
    generateSection(prop, name) {
        const sections = {
            title: prop.name ? [headingNode(prop.name, 2)] : undefined,
            description: prop.description
                ? [
                    paragraphNode(prop.description.split('\n').map((d) => ({
                        kind: NodeKind.Text,
                        value: d,
                    }))),
                ]
                : undefined,
            type: this.getType(prop),
            location: this.getLocation(prop),
            props: this.getPropsTable(prop),
            extends: this.getExtends(prop),
            examples: this.getExamples(prop),
        };
        const config = this.sections[name];
        if (config) {
            const { title, render } = config;
            const children = typeof render === 'function' ? render(prop) : sections[name];
            if (children) {
                const result = [];
                if (title) {
                    const value = typeof title === 'function' ? title(prop) : title;
                    if (value) {
                        result.push(paragraphNode([boldNode([textNode(value)])]));
                    }
                }
                result.push(...children);
                return result;
            }
        }
        return undefined;
    }
    generateNodes(prop) {
        const result = [];
        Object.keys(this.sections).forEach((name) => {
            const config = this.sections[name];
            if (!(config === null || config === void 0 ? void 0 : config.hidden)) {
                const section = this.generateSection(prop, name);
                if (section) {
                    result.push(...section);
                }
            }
        });
        return result;
    }
    extract(props, options = {}) {
        const result = [];
        const { collapsed = [], extensions, visible, sections = {}, columns = ['all'], skipInherited = false, overrides = {}, } = options;
        this.collapsed = collapsed;
        if (sections) {
            this.sections = Array.isArray(sections)
                ? sections.reduce((acc, name) => {
                    return { ...acc, [name]: defaultSections[name] };
                }, {})
                : deepmerge(defaultSections, sections);
        }
        this.columns = columns;
        this.skipInherited = skipInherited;
        const filterProps = (prop) => {
            const transform = (p) => {
                var _a;
                if (p.name && overrides[p.name] && hasProperties(p)) {
                    const o = overrides[p.name];
                    return {
                        ...p,
                        properties: (_a = p.properties) === null || _a === void 0 ? void 0 : _a.map((pp) => pp.name && o[pp.name] ? { ...pp, ...o[pp.name] } : pp),
                    };
                }
                return p;
            };
            if (Array.isArray(extensions)) {
                if (typeof prop.extension === 'string' &&
                    extensions.includes(prop.extension)) {
                    return transform(prop);
                }
                return undefined;
            }
            if (Array.isArray(visible)) {
                if (typeof prop.name === 'string' && visible.includes(prop.name)) {
                    return transform(prop);
                }
                return undefined;
            }
            return transform(prop);
        };
        Object.keys(props).forEach((key) => {
            if (key !== '__helpers' && key !== '__diagnostics') {
                const prop = filterProps(props[key]);
                if (prop) {
                    this.topLevelProps[key] = prop;
                }
            }
        });
        this.helpers = props.__helpers || {};
        Object.keys(this.helpers).forEach((key) => {
            const prop = filterProps(this.helpers[key]);
            if (prop) {
                this.topLevelProps[key] = prop;
            }
        });
        Object.values(this.topLevelProps).forEach((prop) => {
            const nodes = this.generateNodes(prop);
            result.push(...nodes);
        });
        return result;
    }
}

/**
 * Creates a list of api documentation nodes
 * @param props properties parsed from `structured-types/api`
 * @param options page generation options
 * @returns a list of documentation nodes
 */
const propsToDocumentation = (props, options) => {
    const extractor = new PropsToDocumentation();
    return extractor.extract(props, options);
};
/**
 * Read the api-docs configuration file
 * @param fileName the file that is being analyzed, will be used the starting folder to search for configuration files.
 * @param configFileName pass directly the configuration file name
 * @returns page generation options from the config file
 */
const apiDocsConfig = (fileName, configFileName) => {
    const configExplorer = cosmiconfigSync('api-docs');
    if (configFileName) {
        return configExplorer.load(configFileName);
    }
    const searchPath = path.dirname(fileName);
    return configExplorer.search(searchPath);
};

export { NodeKind, apiDocsConfig, getRepoPath, isBoldNode, isCodeNode, isEmphasisNode, isHeadingNode, isInlineCodeNode, isLinkNode, isNodeWithChildren, isNodeWithValue, isParagraphNode, isTableCellNode, isTableNode, isTableRowNode, isTextNode, propsToDocumentation };
